# xDevs.com Python A3 ADC health test GPIB app for three 3458A 
# (C) 2024 xDevs.com | Illya Tsemenko
# Useful information:
#   https://xdevs.com/guide/life_with_3458/
#   https://xdevs.com/guide/ni_gpib_rpi/
#   https://xdevs.com/fix/hp3458
#   https://xdevs.com/guide/e5810a/

'''
This application runs ACAL every hour on tandem of three HP/Agilent/Keysight 3458A DMMs to verify stability of ADC A3 assembly
Key calibration constants are stored into datafile in same directory as program for further analysis
Deviation of CAL? 72 constant from first day of data by more than 0.02 ppm/day can be considered faulty/bad A3, when not taking temperature coefficient into account.
Drift over 0.05 ppm/day is a telltale of the faulty A3 U180 hybrid ASIC chip. This can be further confirmed by INL sweep on 12V range. 
Some more details explained here: https://xdevs.com/fix/hp3458a/#a3drift, https://xdevs.com/fix/hp3458a/#sn18test
'''

import sys
import time
from colorama import just_fix_windows_console
just_fix_windows_console()                 # Activate support for colors in windows console

# Setting variables
dmm1_gpib_addr = 5
dmm2_gpib_addr = 4
dmm3_gpib_addr = 3
filename1 = ("sn18_n3458a_gpib_%d_2024data.csv" % dmm1_gpib_addr)
filename2 = ("sn18_n3458b_gpib_%d_2024data.csv" % dmm2_gpib_addr)
filename3 = ("sn18_n3458c_gpib_%d_2024data.csv" % dmm3_gpib_addr)
vxi11_ip_addr = "192.168.50.14"
hw_interface_type = "vxi"                  # Please select vxi or visa or linux-gpib
timegap = 3600                             # Number of seconds to wait between ACALs
run_hours = 720                            # Run 30 days for stability data (24 * 30 day = 720 hours)

print ("\033[44;33m -n- SN18 verification data collection script for 3 x HP3458A. (C) xDevs.com 2024\033[49;39m")

if (hw_interface_type == "vxi"):
    import vxi11                           # Import library for Agilent E5810A/B GPIB to LAN bridge
elif (hw_interface_type == "visa"):
    import pyvisa                          # Import library for VISA interface
elif (hw_interface_type == "linux-gpib"):
    import Gpib                            # Import library for USB linux-gpib (works only on Linux)
else:
    print ('\033[31;1m -e- Incorrect interface define, use "vxi" or "visa" or "linux-gpib" for hw_interface_type parameter\033[39;0m')

# Function to initialize GPIB communication
def intf(gpib_num):
    if (hw_interface_type == "vxi"):
        try:
            return vxi11.Instrument(vxi11_ip_addr, "gpib0,%d" % gpib_num)
        except:
            print ('\033[31;1m -e- Cound not initialize GPIB-LAN bridge at %s for GPIB %d\033[39;0m' % (vxi11_ip_addr, gpib_num))
    elif (hw_interface_type == "visa"):
        rm = pyvisa.ResourceManager()
        #rm.list_resources()   # Debug listing for all available interfaces
        return rm.open_resource('GPIB::%d"::INSTR' % gpib_num)
    elif (hw_interface_type == "linux-gpib"):
        return Gpib.Gpib(0,gpib_num,timeout = 60)

inst = intf(dmm1_gpib_addr)
inst.timeout = 30
inst.clear()
insta = intf(dmm2_gpib_addr)
insta.timeout = 30
insta.clear()
instb = intf(dmm3_gpib_addr)
instb.timeout = 30
instb.clear()

# Initialize first 3458A
inst.write("RESET")
inst.write("END ALWAYS")
inst.write("PRESET NORM")
inst.write("OFORMAT ASCII")
inst.write("ID?")
data = inst.read()
print ("\033[32;1m -i- %s detected on GPIB %d \033[39;0m" % (data, dmm1_gpib_addr))

# Initialize second 3458A
insta.write("RESET")
insta.write("END ALWAYS")
insta.write("PRESET NORM")
insta.write("OFORMAT ASCII")
insta.write("ID?")
data = insta.read()
print ("\033[32;1m -i- %s detected on GPIB %d \033[39;0m" % (data, dmm2_gpib_addr))

# Initialize third 3458A
instb.write("RESET")
instb.write("END ALWAYS")
instb.write("PRESET NORM")
instb.write("OFORMAT ASCII")
instb.write("ID?")
data = instb.read()
print ("\033[32;1m -i- %s detected on GPIB %d \033[39;0m" % (data, dmm3_gpib_addr))

# Function to read calibration values BEFORE running ACAL
def log_data_pre(ins, files, id):
    ins.write("TEMP?")
    data = ins.read()
    print ("\033[37;1m -i- %s internal TEMP? = %s C\r\033[39;0m" % (id, data))
    ins.write("CAL? 1,1")
    data = ins.read()
    ins.write("CAL? 2,1")
    data = ins.read()
    ins.write("CAL? 72")
    data = ins.read()
    
# Function to read calibration values AFTER successful ACAL ALL
def log_data_after(ins, files, id):
    ins.write("TEMP?")
    data = ins.read()
    print ("\033[37;1m -i- %s internal TEMP? = %s C\r\033[39;0m" % (id, data))
    files.write (time.strftime("%d/%m/%Y-%H:%M:%S;") + (("TEMP? = ") + data + "|"))
    ins.write("CAL? 1,1")
    data = ins.read()
    files.write (time.strftime("%d/%m/%Y-%H:%M:%S;") + (("CAL? 1,1 = ") + data  + "|"))
    ins.write("CAL? 2,1")
    data = ins.read()
    files.write (time.strftime("%d/%m/%Y-%H:%M:%S;") + (("CAL? 2,1 = ") + data + "|"))
    ins.write("CAL? 78")
    data = ins.read()
    files.write (time.strftime("%d/%m/%Y-%H:%M:%S;") + (("CAL? 78 R10kG = ") + data + "|"))
    ins.write("CAL? 79")
    data = ins.read()
    files.write (time.strftime("%d/%m/%Y-%H:%M:%S;") + (("CAL? 79 R100kG = ") + data + "|"))
    ins.write("CAL? 71")
    data = ins.read()
    files.write (time.strftime("%d/%m/%Y-%H:%M:%S;") + (("CAL? 71 DCV1G = ") + data + "|"))
    ins.write("CAL? 70")
    data = ins.read()
    files.write (time.strftime("%d/%m/%Y-%H:%M:%S;") + (("CAL? 70 DCV100mG = ") + data + "|"))
    ins.write("CAL? 86")
    data = ins.read()
    files.write (time.strftime("%d/%m/%Y-%H:%M:%S;") + (("CAL? 86 R1kGO = ") + data + "|"))
    ins.write("CAL? 87")
    data = ins.read()
    files.write (time.strftime("%d/%m/%Y-%H:%M:%S;") + (("CAL? 87 R10kGO = ") + data + "|"))
    ins.write("CAL? 176")
    data = ins.read()
    files.write (time.strftime("%d/%m/%Y-%H:%M:%S;") + (("CAL? 176 ACAL TEMP = ") + data + "|"))
    ins.write("CAL? 59")
    data = ins.read()
    files.write (time.strftime("%d/%m/%Y-%H:%M:%S;") + (("CAL? 59 CAL 10V TEMP = ") + data + "|"))
    ins.write("CAL? 97")
    data = ins.read()
    files.write (time.strftime("%d/%m/%Y-%H:%M:%S;") + (("CAL? 97 DCI1mAG = ") + data + "|"))
    ins.write("CAL? 72")
    data = ins.read()
    print ("\033[35;1m -i- %s CAL? 72 value = %s \r\033[39;0m" % (id, data))
    files.write (time.strftime("%d/%m/%Y-%H:%M:%S;") + (("CAL? 72 = ") + data + "|\r"))
    # Beep multiple times for user awareness
    ins.write("BEEP")
    time.sleep(0.1)
    ins.write("BEEP")
    time.sleep(0.5)
    ins.write("BEEP")
    time.sleep(1)
    ins.write("BEEP")

for cycles in range(0,run_hours): # Perform hours sequence and append each run data into files
    with open(filename1, 'a') as o1:
        with open(filename2, 'a') as o2:
            with open(filename3, 'a') as o3:
                # Collect pre-ACAL data
                log_data_pre(inst,  o1, "3458A")
                log_data_pre(insta, o2, "3458B")
                log_data_pre(instb, o3, "3458C")

                # Execute ACAL ALL on all instruments
                inst.write("ACAL ALL")
                insta.write("ACAL ALL")
                instb.write("ACAL ALL")
                time.sleep(860)     # Wait for 860 seconds while ACAL ALL is running

                # Collect key CAL constants after ACAL done
                log_data_after(inst,  o1, "3458A")
                log_data_after(insta, o2, "3458B")
                log_data_after(instb, o3, "3458C")
            o3.close()
        o2.close()
    o1.close()

    print ("\033[36;1m -i- ACAL cycle %d done \r\033[39;0m" % (cycles))
    time.sleep(timegap - 860)    # wait for next hour to execute next ACAL iteration
    
print ("All done! Check for more at https://xDevs.com")